import { escapeRegExp } from "lodash";
import resolve from "resolve";

export function moduleResolverByPaths(root, paths, extensions) {
    const { resolvePath } = require("babel-plugin-module-resolver");
    const matchers = createMatchers(paths);
    const options = {
        root,
        extensions,
        resolvePath(specifier, importer, options) {
            let path;
            for (let i = 0; !path && i < matchers.length; i++)
                path = tryMatcher(matchers[i], resolvePath, specifier, importer, options);
            return path || resolvePath(specifier, importer, options);
        },
    };
    return ["babel-plugin-module-resolver", options, "ts-config-paths-module-resolver"];
}

function tryMatcher(matcher, resolver, specifier, importer, options) {
    let path;
    const [regex, substitutions] = matcher;
    const [, value] = regex.exec(specifier) || [];
    if (value) {
        const context = {
            basedir: options.root,
            extensions: options.extensions,
        };
        for (let i = 0; !path && i < substitutions.length; i++) {
            const name = substitutions[i].replace("*", value);
            path = tryResolve(name, context);
        }
    }
    return path;
}

function createMatchers(paths) {
    const matchers = [];
    for (const key of Object.keys(paths)) {
        const pattern = escapeRegExp(key).replace("\\*", "(.+)");
        const regex = new RegExp(pattern);
        const substitutions = paths[key];
        matchers.push([regex, substitutions]);
    }
    return matchers;
}
function tryResolve(name, options) {
    try {
        return resolve.sync(name, options);
    }
    catch (e) {
    }
}
//# sourceMappingURL=module-resolver.js.map
