Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@babel/core");
const core_2 = require("@webpackery/core");
const path_1 = require("path");
const module_resolver_1 = require("./module-resolver");
const ts_utils_1 = require("./ts-utils");
class BabelConfigurer extends core_2.WebpackConfigurer {

    prepare(props, context) {
        const { production } = context;

        props = Object.assign({}, props);

        props.options = Object.assign({ sourceMaps: production, envName: production ? "production" : "development", overrides: [] }, props.options);

        props.options.overrides = Array.from(props.options.overrides);
        const config = core_1.loadPartialConfig(Object.assign({ filename: path_1.resolve("./any.js"), caller: {
                name: "@webpackery/babel-configurer",
                supportsStaticESM: true,
                supportsDynamicImport: true,
            } }, props.options));
        core_2.setDefaultBy(props, "extensions", resolveDefaultExtensions, config);
        core_2.setDefaultBy(props, "test", extensionsToRegex, props.extensions);
        if (isTypeScript(config)) {
            const ts = require("typescript");
            const tsconfigPath = ts_utils_1.findTsConfigFile(ts, context.config.context);
            if (tsconfigPath) {
                const tsconfig = ts_utils_1.readTsConfigFile(ts, tsconfigPath);
                const { paths, baseUrl } = tsconfig.compilerOptions;
                if (paths) {
                    const root = path_1.resolve(path_1.dirname(tsconfigPath), baseUrl);
                    const item = module_resolver_1.moduleResolverByPaths(root, paths, props.extensions);
                    const override = { plugins: [item] };
                    props.options.overrides.push(override);
                }
            }
        }
        return props;
    }

    configure(context, props) {
        const { extensions } = props, rule = tslib_1.__rest(props, ["extensions"]);
        context.module.rule(Object.assign({ loader: "babel-loader" }, rule));
        context.resolve.extension(...extensions);
    }
}
exports.BabelConfigurer = BabelConfigurer;
function extensionsToRegex(extensions) {
    if (extensions && extensions.length) {
        const group = extensions.map(x => x.slice(1)).join("|");
        const pattern = `\.(${group})$`;
        return new RegExp(pattern);
    }
}
function resolveDefaultExtensions(config) {
    const react = isReact(config);
    const typescript = isTypeScript(config);
    return [
        ".js", ".es6", ".es", ".mjs",
        react && ".jsx",
        typescript && [".ts", ".d.ts"],
        typescript && react && ".tsx",
    ].filter(Boolean).flat();
}
function isReact(config) {
    return hasFile(config, "@babel/preset-typescript");
}
function isTypeScript(config) {
    return hasFile(config, "@babel/preset-typescript");
}
function hasFile(config, preset) {
    const items = config.options.presets;
    return containsFileRequest(items, preset);
}
function containsFileRequest(items, preset) {
    if (items)
        return items.some(item => item.file && item.file.request === preset);
}
//# sourceMappingURL=babel-configurer.js.map
